/**
 * This plugin creates a random mcq question with two option to compare more or less number of objects
 * @class compobject
 * @extends Plugin
 * @author Devendra Singh <devendra.singh@tarento.com>
 * @fires invoke
 * @fires invokeChildren
 */
Plugin.extend({
    _type: 'compobject',
    _isContainer: true,
    _render: true,  
    _itemData: undefined,
    _quesTypes:["more","less"], //two different types of questions
    _objectTypes: ["familiar","geometric","number"], // three different categories of objects
    _familiarImages: [],
    _geometricImages: [],
    _numberImages: [],
    _totalOptions: undefined,
    _quesType: undefined,
    _selectedImage: undefined,
    _selectedObjectType: undefined,
    _quesIndex:undefined,
    _differenceCount:undefined,
    _maxCount:undefined,
    _quesText: undefined,
    _quesLang: undefined,
    initPlugin: function(data) {
        this._quesText={
            "hi": {"lang": "Hindi", "less": "जिस बक्से में कम वस्तुएँ हैं उस पर क्लिक कीजिए:", "more": "जिस बक्से में ज्यादा वस्तुएँ हैं उस पर क्लिक कीजिए:"},
            "en": {"lang": "English","less": "Choose the box that has LESS objects:", "more": "Choose the box that has MORE objects:"},
            "mar": {"lang": "Marathi", "less": "सर्वात कमी वस्तू असलेली चौकट निवडा:", "more": "सर्वात जास्त वस्तू असलेली चौकट निवडा:"},
            "te": {"lang": "Telugu", "less": "ఏవి తక్కువ ఉన్నాయి?", "more": "ఏవి ఎక్కువ ఉన్నాయి?"}
        }
        this._quesIndex= this._stage._stageController._index;
        this._totalOptions= 2;
        this.addDefaultImages(data);
        var model= data.model;
        var dims= this.relativeDims();
        this._self= new createjs.Container();
        this._self.x= dims.x;
        this._self.y= dims.y;
        this._itemData= this._stage.getController(model);
        if(!this._itemData._model[this._quesIndex].isItemSet){
            this._maxCount= _.random(4,10);
            this._selectedObjectType= this._stage.getModelValue(data.objType);
            // this._differenceCount= this._stage.getModelValue(data.diff);
            this._differenceCount= "<=3";
            this._quesLang= this._stage.getModelValue(data.quesLang);
            this._quesLang= this._quesLang.toLowerCase();
            this._selectedImage= this.getOptionImage();
            this.updateItemData();
            this._itemData._model[this._quesIndex].isItemSet= true;
        }
        data.mcq= this.getMcqTemp();
        var quesText;
        quesText= this.getQuesTextTemp();
        PluginManager.invoke("text",quesText,this._stage,this._stage, this._theme);
        this.invokeChildren(data, this, this._stage, this._theme);
    },
    /**
     * This method prepares question object and set it to Controller
     * @memberof compobject
     * @fire {function} setQuesOptions, sets the options of the question
     */
    updateItemData: function(){
        var quesType = this._quesType = this.getQuesType();
        var ques= this._quesText[this._quesLang][this._quesType];
        this._itemData._model[this._quesIndex].question= ques;
        this.setQuesOptions();
        Renderer.update= true;
    },
     /**
     * This method sets default images in the _images arary
     * @memberof compobject
     */
    addDefaultImages: function(data){
        this._familiarImages= ["apple","ball","banana","mango","orange"];
        this._geometricImages= ["circle","rectangle","square","triangle"];
        this._numberImages= ["one","ten"];

    },

     /**
     * This method returns a random image from the _images arary
     * @memberof compobject
     * @return {String}, returns a random string which is image id
     */
    getOptionImage: function(){
        switch(this._selectedObjectType.toLowerCase()) {
            case "familiar":
                return _.sample(this._familiarImages);
                break;
            case "geometric":
                return _.sample(this._geometricImages);
                break;
            case "number":
                return _.sample(this._numberImages);
                break;
            default:
                return _.sample(this._familiarImages);
                break;
        }
        
    },

    /**
     * This method returns ques type from _quesTypes array
     * @memberof compobject
     * @return {String}, returns a string which is question type
     */
    getQuesType:function(){
        return _.sample(this._quesTypes);
    },

     /**
     * This method prepares mcq objects 
     * @memberof compobject
     * @return {object} mcq, object which contains options layout
     */
    getMcqTemp: function(){
        var marginX= 3;
        var marginY= 4;
        var mcq={
                "multi_select": "false",
                "model": "item",
                "density": "2",
                "option":[]
                };
        var options= this._itemData._model[this._quesIndex].options;
        var w= 100;
        var totalRows= Math.ceil(this._totalOptions); //totalRows is no of rows to plot the options
        var h= (100-((totalRows-1)*marginY))/totalRows;
        for(var i=0; i< options.length; i++){ //loop to create each option object
            var option={}
            option.x= 0;
            if(i>0){
                option.y= i*h +marginY;   
            }else{
                option.y= i*h;
            }
            option.w= w;
            option.h= h;
            option.option= "options["+i+"]";
            option.shadowColor= "#4c4c4c";
            option.highlight="#4c4c4c";
            option.blur= 30;
            option.shape= {"fill": "#F3F3F3", "x": "0" ,"y": "0", "w":"100", "h":"100", "type": "roundrect", "stroke": "#B7B7B7"};
            option.grid={
                "x": "5",
                "y": "5",
                "w":"90",
                "h":"90",
                "rows":"2",
                "cols":"10",
                "iterate":"",
                "offsetX": "2",
                "count": options[i].value.count,
                "image": {"x":"0", "y":"0", "h":"100", "asset": options[i].value.image,"align": "top", "valign":"middle"}
            };
            mcq.option.push(option);
        }
        return mcq;   
    },
    /**
     * This method returns ques text template
     * @memberof compobject
     * @return {object}, returns a object which is question text template
     */
    getQuesTextTemp: function(){
        return  {"model": "item.question", "x": "12", "y": "10", "w": "76", "color": "#4c4c4c", "fontsize": "3vw", "align": "center", "valign": "top", "lineHeight": "1.4" };
    },

    /**
     * This method creats the options for question and updates them to item controller
     * @memberof compobject
     * @param {number} totalOptions, no of option added for the perticular question
     */
    setQuesOptions: function(){
        var totalOptions= this._totalOptions;
        this._itemData._model[this._quesIndex].options= [];
        var options=  this._itemData._model[this._quesIndex].options;
        for(var i=0; i< totalOptions; i++){
                if(_.isUndefined(options[i])){
                    var count= this.getOptionCount(i);
                    options[i]={};
                    options[i].selected= undefined;
                    options[i].value={ "type" : "mixed", "image": this._selectedImage , "count": count}  
                }

        }
        if(this._quesType == "more"){
            if(options[0].value.count> options[1].value.count){
                options[0].answer= true;
            }else{
                options[1].answer= true;
            }
        }else{
            if(options[0].value.count< options[1].value.count){
                options[0].answer= true;
            }else{
                options[1].answer= true;
            }
        }

        this._itemData._model[this._quesIndex].options= this.registerResValues(options);
        this._itemData._model[this._quesIndex].options= _.shuffle(this._itemData._model[this._quesIndex].options);

    },
    /**
     * This method updates resindex and resvalue to each option object
     * @memberof compobject
     * @param {array} options, options array of the question
     * @return {array} options, options with resindex and resvalue for the question
     */
    registerResValues: function(options) {
        _.each(options, function(option, index) {
          if(typeof option.value.resvalue == 'undefined'){
                option.value.resvalue = option.value.asset || option.value.text || option.value.image || option.value.count || option.value.audio || 'option' + index;  

            }
            option.value.resindex = index;
        });
        return options;
    },
    /**
     * This method returns count to repeat images in option
     * @memberof compobject
     * @return {number} count, it is the number which will be used in respective option to add images
     */
    getOptionCount: function(index){
        if(index){
            var count,diff;
            if(this._differenceCount == "<=3"){
                diff= _.sample([1,2,3]);
            }else{
                diff= _.sample([3,4,5,6]);
            }
            count= this._maxCount-diff;
            if(count <=0){
                count= this._maxCount+diff;
            }
            if(count >10){
                count= this._maxCount-diff;
            }
            if(count>0 && count<=10 ){
                return count;
            }else{
                this.getOptionCount(index);
            }
        }else{
            return this._maxCount;
        }
    }

});

